/** @mainpage countSmarts
* <H1><center><B>countSmarts</B></center></H1>
* <p>
* @b countSmarts counts the occurrences of SMARTS in a SMILES. Both the SMARTS and the SMILES have to
* be provided as tab separated files with SMARTS/SMILES in the first column and an optional name in
* the second column. The program also allows the removal of duplicate SMILES where duplicate means that
* the unique SMILES are identical.
* <p>
* &copy; 2004 by Uli Fechner
*/

/** @file
*
* This is the main file of the program countSmarts. All other files are included here via the inclusion
* of the file @ref includes.h.
*
* @author Uli Fechner
* @version 11/06/2004 - UF - 0.1 - initial stable release
*/

/* preprocessor includes */

#include "includes.h"

/* just some information about this file */
#define COUNTSMARTS_VERSION	"0.1"
#define COUNTSMARTS_DATE	"11-JUN-2004"

/* function prototypes */

int main( int argc, char *argv[] );

void parseClp( const int argc, char* argv[], CLP_Ptr clpPtr );

List_Ptr readDataFromFile( const char* const filename, const int transform, const int uniqueData, \
	FILE* errorLogFile );

List_Ptr readDataFromStream( FILE* inputStream, const char* const nameOfStream, \
	const int daylight_type, const int uniqueData, FILE* errorLogFile );

DoubleArrayPtr getFileProperties( FILE* inputStream, const char* const nameOfStream );

void displayHelpText( );

void displayVersionInformation( void );

/* functions */

/** Main function of the program.
*
* The main function just calls the other functions. It is reduced to contain only function calls
* whenever possible.
*
* @param argc the number of arguments provided via the command line by executing the program
* @param *argv[] string array containing the command line arguments
* @retval int exit code of the program
* @author Kristina Grabowski
* @version 20/02/2004 - Uli Fechner - 0.1 - initial stable release
*/
int main ( int argc, char *argv[] )
{
	CLP_Ptr clpPtr; /* pointer on CommandLineParameters */
	FILE* outFile; /* file pointer on the output file */
	FILE* errorLogFile; /* file pointer on the SMILES error log file */
	List_Ptr inputDataPtr; /* pointer on a structure List that stores the data of the input file */
	List_Ptr smartsDataPtr; /* pointer on a List that stores the data of the reaction smiles file */
	SmilesCompound_Ptr currentSmartsPtr; /* pointer on current reaction SMILES in OUTER LOOP */
	SmilesCompound_Ptr currentSmilesPtr; /* pointer on current input SMILES in INNER LOOP */
	int totalNumberOfCompounds = 0; /* total number of compounds in input file */
	int matchcounter = 0;	/* number of matched SMARTS for all SMARTS in the input SMART file */
	
	/***** Daylight Toolkit variables *****/
	dt_Handle errorSequenceHandle; /* dt_Handle on a stream that contains the error messages of the DT */
	dt_Handle errorHandle; /* dt_Handle on one error message of errorSequenceHandle */
	dt_Integer length; /* needed by dt_cansmiles, dt_typename and dt_stringvalue */
	/* Daylight Toolkit variables of the OUTER LOOP */
	dt_Handle currentSmartsHandle; /* dt_Handle on current reaction SMILES */
	/* Daylight Toolkit variables of the INNER LOOP */
	dt_Handle currentSmilesHandle; /* dt_Handle on the current input SMILES */
	/* dt_Handle on the pathset, each path in the pathset contains a unique set of atoms */
	dt_Handle pathset;
	/* dt_Handle on a stream object containing all of the parts of type 'TYP_PATH' in the object 'pathset'*/
	dt_Handle pathStream;
	dt_Handle path; /* dt_Handle on the current path */    
	
	/* creating a CommandLineParameters structure and initialize it with default values */
	clpPtr = CLP_create( );
	parseClp( argc, argv, clpPtr ); /* parsing the command line arguments */
	/* the command line arguments are shown on standard error */
	fprintf( stderr, "\ncountSmarts %s ( %s )\n\n", COUNTSMARTS_VERSION, COUNTSMARTS_DATE );
	CLP_display( clpPtr, stderr );
	
	/* opening the output file; if an error occurs the program aborts */
	if( !( outFile = fopen( CLP_getOutputFile( clpPtr ), "w" ) ) )
		FileWriteError( CLP_getOutputFile( clpPtr ) );
	
	/* opening the SMILES error log file; if an error occurs the program aborts */
	if( !( errorLogFile = fopen( CLP_getErrorLogFile( clpPtr ), "w" ) ) )
		FileWriteError( CLP_getErrorLogFile( clpPtr ) );

	/* reading the data from stdin */
	fprintf( stderr, "Reading file from stdin......." );
	inputDataPtr = readDataFromStream( stdin, "<stdin>", DAYLIGHT_SMILES, \
		CLP_getUniqueSmiles( clpPtr ), errorLogFile );
	List_setName( inputDataPtr, "SMILESFile" );
	List_rewind( inputDataPtr );
	fprintf( stderr, "Done\n" );
	while( List_hasNext( inputDataPtr ) )
		totalNumberOfCompounds += SmilesCompound_getCounter( List_getNext( inputDataPtr ) );
	fprintf( stderr, "Total number of compounds in input file: %d\n", totalNumberOfCompounds );
	/* print out the number of unique compounds if the option is set */
	if( CLP_getUniqueSmiles( clpPtr ) == BOOLEAN_TRUE )
		fprintf( stderr, "Number of unique compounds in input file: %d\n\n", \
		List_getNumberOfNodes( inputDataPtr ) );

	/* reading SMARTS (provided with '-s' option) */
	fprintf( stderr, "Reading SMARTES file %s...", CLP_getSmartsFile( clpPtr ) );
	smartsDataPtr = readDataFromFile( CLP_getSmartsFile( clpPtr ), DAYLIGHT_SMARTS, \
		CLP_getUniqueSmiles( clpPtr ), errorLogFile );
	List_setName( smartsDataPtr, "SMARTSFile" );
	fprintf( stderr, "Done\n" );
	fprintf( stderr, "Number of SMARTS: %d\n", List_getNumberOfNodes( smartsDataPtr ) );
	
	/* printing the header row */
	fprintf( outFile, "#ID" );
	List_rewind( smartsDataPtr );
	while( List_hasNext( smartsDataPtr ) )
	{
		currentSmartsPtr = List_getNext( smartsDataPtr );
		if( StringArray_getNumberOfElements( SmilesCompound_getStringArrayOfNames( currentSmartsPtr ) ) > 0 )
			fprintf( outFile, "\t%s", StringArray_getElement( SmilesCompound_getStringArrayOfNames( \
				currentSmartsPtr ), 0 ) );
		else
			fprintf( outFile, "\t%s", SmilesCompound_getSmiles( currentSmartsPtr ) );
	}
	fprintf( outFile, "\n" );
	
	/* rewinding both lists */
	List_rewind( inputDataPtr );
	List_rewind( smartsDataPtr );
	fprintf( stderr, "Counting SMARTS in SMILES..." );
	/***** OUTER LOOP: LOOPING OVER THE SMILES FILE *****/
	while( List_hasNext( inputDataPtr ) )
	{
		currentSmilesPtr = List_getNext( inputDataPtr );
		currentSmilesHandle = SmilesCompound_getMoleculeHandle( currentSmilesPtr );
		fprintf( outFile, "%s", StringArray_getElement( \
			SmilesCompound_getStringArrayOfNames(currentSmilesPtr), 0 ) );

		/***** INNER LOOP: LOOPING OVER THE SMARTS FILE *****/	
		while( List_hasNext( smartsDataPtr ) )
		{
			/* counts the matches for one type of SMARTS in the SMARTS file */
			matchcounter = 0;
			currentSmartsPtr = List_getNext(smartsDataPtr);
			currentSmartsHandle = SmilesCompound_getMoleculeHandle(currentSmartsPtr);
			
			/* choose the correct SMARTS matching method specified via a command line parameter */
			switch( CLP_getMatchingType( clpPtr ) )
			{
				case 1:
					pathset = dt_match( currentSmartsHandle, currentSmilesHandle, 0 );
					break;
				case 2:
					pathset = dt_umatch( currentSmartsHandle, currentSmilesHandle, 0 );
					break;
				case 3:
					pathset = dt_xmatch( currentSmartsHandle, currentSmilesHandle, 0 );
					break;
				default:
					pathset = dt_umatch( currentSmartsHandle, currentSmilesHandle, 0 );
					break;
			}
			
			if( pathset != NULL_OB )
			{ 
				pathStream = dt_stream( pathset, TYP_PATH );
				while( NULL_OB != ( path = dt_next( pathStream ) ) ) 
					matchcounter++;
				dt_dealloc( pathset ); /* cleaning up */
			}
			fprintf( outFile, "\t%d", matchcounter );
    }
		/**** END OF INNER LOOP ****/
		fprintf(outFile, "\n");
		List_rewind( smartsDataPtr );

	}
	/**** END OF OUTER LOOP ****/
	fprintf( stderr, "Done\n" );
	
	/* writing the Daylight Toolkit errors to the error log file */
	errorSequenceHandle = dt_errors( DX_ERR_NONE );
	while( NULL_OB != ( errorHandle = dt_next( errorSequenceHandle ) ) )
		fprintf( errorLogFile, "%s\n", dt_stringvalue( &length, errorHandle ) );
	
	/* cleaning up */
	List_destroy( inputDataPtr );
	List_destroy( smartsDataPtr );
	fclose( outFile );
	CLP_destroy( clpPtr );
	fprintf( stderr, "Adios.\n\n");
	return EXIT_SUCCESS;
}

/** Parses the command line parameters.
*
* The command line parameters are parsed here. Additionally, it is checked whether the standard in is
* connected to a terminal. If this is the case, no input file is provided and the help text of the
* program is printed on standard error. It is also checked here if there are any incompatibilities
* of the options provided at the command line. If so, the program aborts with an appropriate error
* message.
*
* @param argc the number of arguments provided via the command line by executing the program
* @param *argv[] string array containing the command line arguments
* @param clpPtr pointer on structure @ref CommandLineParameters that is used to store all the command line parameters
* @author Kritsina Grabowski
* @version 20/02/04 - Uli Fechner - initial release
*/
void parseClp( const int argc, char* argv[], CLP_Ptr clpPtr )
{	
	int c; /* gets the arguments via getopt */
	int matchingType; /* intermediate storage of the matching type as a number */
	char* errorLogFile; /* temporary storage of the error log file name */
	Given_CLP_Ptr givenClpPtr; /* pointer on structure GivenCommandLineParameters */	

	givenClpPtr = GivenClp_create( ); /* creating and initializing a structure GivenCommandLineParameters */
	
	opterr = 0; /* disable the error message getopt normaly prints on stderr */
	
	while ( ( c = getopt( argc, argv, "hvuo:s:m:" ) ) != -1 )
	{
		switch( c )
		{
		/* checking if to provide the help text */
		case 'h':
			displayHelpText( );
			break;
		/* checking if to provide version information text */
		case 'v':
			displayVersionInformation();
			break;
		/* the argument of the argument o defines the name of the output file */
		case 'o':
			CLP_setOutputFile( clpPtr, optarg );
			GivenClp_setOption( givenClpPtr, 'o', BOOLEAN_TRUE );
			break;		
		/* the argument of the argument s defines the name of the smartsfile */
		case 's':
			CLP_setSmartsFile( clpPtr, optarg );
			GivenClp_setOption( givenClpPtr, 's', BOOLEAN_TRUE );
			break;
		case 'm':
			sscanf( optarg, "%d", &matchingType );
			CLP_setMatchingType( clpPtr, matchingType );
			GivenClp_setOption( givenClpPtr, 'm', BOOLEAN_TRUE );
			break;
		case 'u':
			CLP_setUniqueSmiles( clpPtr, BOOLEAN_TRUE );
			GivenClp_setOption( givenClpPtr, 'u', BOOLEAN_TRUE );
			break;
		case '?':
			fprintf( stderr, "\n\nERROR: There is given either an option without an argument that\n" );
			fprintf( stderr, "requires one or an unknown option!\n" );
			fprintf( stderr, "Type 'countSmarts -h' for a detailed help text!\n");
			AbortProgram;
			break;					
		}
	}
	
	/* testing whether the standard in is connected to the terminal; if so display help */
	if( isatty( STDIN_FILENO ) )
		displayHelpText( clpPtr );

	/* the '-o' argument is mandatory; if itisn't provided the program aborts with an appropriate message */
	if( GivenClp_getOption( givenClpPtr, 'o' ) == BOOLEAN_FALSE )
		MandatoryOption( "'-o'" );

	/* the '-s' argument is mandatory; if itisn't provided the program aborts with an appropriate message */
	if( GivenClp_getOption( givenClpPtr, 's' ) == BOOLEAN_FALSE )
		MandatoryOption( "'-s'" );
	
	/* generating the name of the error log file and storing it in the structure CommandLineParameters */
	if( !( errorLogFile = calloc( strlen( CLP_getOutputFile( clpPtr ) ) + 5, sizeof( char ) ) ) )
		MemoryError( "parseClp", "errorLogFile" );
	strncpy( errorLogFile, CLP_getOutputFile( clpPtr ), strlen( CLP_getOutputFile( clpPtr ) ) + 1 );
	CLP_setErrorLogFile( clpPtr, strncat( errorLogFile, ".log", 4 ) );
	if( errorLogFile != NULL )
		free( errorLogFile );
	
	GivenClp_destroy( givenClpPtr ); /* destroying structure GivenCommandLineParameters */
}

/** Reads the data of a file and stores it in a structure @ref List.
* 
* @attention
* This is mainly a wrapper for function @ref readDataFromStream!
*
* @param filename the name of the file to read
* @param daylight_type indicates the file type (@ref DAYLIGHT_SMILES, @ref DAYLIGHT_SMIRKS, @ref DAYLIGHT_SMARTS)
* @param uniqueData if set to @ref BOOLEAN_TRUE input SMILES are filtered to yield unique SMILES
* @param errorLogFile file pointer on the error log file
* @retval List_Ptr points on structure @ref List containing the content of @c filename
* @author Uli Fechner
* @version 28/11/2003 - Uli Fechner - initial release
* @version 09/06/2004 - UF - added new parameter @c uniqueData
*/
List_Ptr readDataFromFile( const char* const filename, const int daylight_type, const int uniqueData, \
	FILE* errorLogFile )
{
	FILE* dataFile; /* file pointer of the file with 'filename' */

	/* opening of the file and checking if this was successful */
	if( !( dataFile = fopen( filename, "r" ) ) )
		FileReadError( filename );
	
	/* calling of the sub-function that actually reads the file */
	return readDataFromStream( dataFile, filename, daylight_type, uniqueData, errorLogFile );
}

/** Reads the data of a stream and stores it in a structure @ref List.
*
* @param inputStream file pointer of the stream to read
* @param nameOfStream string indicating the name of the stream
* @param daylight_type indicates the file type (@ref DAYLIGHT_SMILES, @ref DAYLIGHT_SMIRKS, @ref DAYLIGHT_SMARTS)
* @param uniqueData if set to @ref BOOLEAN_TRUE input SMILES are filtered to yield unique SMILES
* @param errorLogFile file pointer on the error log file
* @retval List_Ptr points on structure @ref List containing the content of @c inputStream
* @author Uli Fechner
* @version 28/11/2003 - Uli Fechner - initial release
* @version 08/12/2003 - Uli Fechner - considered unsuccessful creations of SmilesCompounds due to
* invalid input SMILES; added error output
* @version 09/06/2004 - UF - added new parameter @c uniqueData; bugfix: SMILES without a name were
*                     assigned an empty string;
* @version 11/06/2004 - UF - MAJOR change: from now on the first column is the SMILES and the second
*                     column is the name (if any; before the 1st column was the name and the 2nd column
*                     was the SMILES); this follows the common format of SMILES files
*/
List_Ptr readDataFromStream( FILE* inputStream, const char* const nameOfStream, const int daylight_type, \
	const int uniqueData, FILE* errorLogFile )
{
	int numberOfColumns; /* the number of columns in the file provided via 'inputStream' */
	/* maximum number of characters per column of the file provided via 'inputStream' */
	int maxNumberOfCharsPerRow;
	int numberOfRows; /* the number of rows in the file provided via 'inputStream' */
	char* tempRow; /* temporary storage of a row of 'inputStream' */
	char* substring; /* pointer needed by the algorithm that divides a line into columns */
	char* tempSmiles; /* temporary storage for the smiles of the current row */
	char* tempName = NULL; /* temporary storage for the name of the current row */
	DoubleArrayPtr dAPtr; /* pointer on DoubleArray containing the results of getFileProperties */
	List_Ptr listPtr; /* pointer on DoubleLinkedList containing the content of the file */
	SmilesCompound_Ptr scPtr; /* temporary pointer on a SmilesCompound during insertion of a compound */
	char* errorMessage; /* for each unsuccessful compound creation a error message is created */
	/* pointer on structure stringArray to store the error messages for unsuccessful compound creations */
	StringArray_Ptr sAPtr;
	
	/* examine the file (number of columns, number of rows, maximum number of chars per row) */
	dAPtr = getFileProperties( inputStream, nameOfStream );
	
	rewind( inputStream );
	numberOfColumns = (int) DoubleArray_getValue( dAPtr, 0 );
	maxNumberOfCharsPerRow = (int) DoubleArray_getValue( dAPtr, 1 );
	numberOfRows = (int) DoubleArray_getValue( dAPtr, 2 );
	
	#if COUNTSMARTS_DEBUG
	fprintf( stdout, "\nNoColumns = %d\tMaxNoChars = %d\tNoRows = %d\n", \
		numberOfColumns, maxNumberOfCharsPerRow, numberOfRows );
	#endif

	if( !( tempRow = calloc( maxNumberOfCharsPerRow + 2, sizeof( char ) ) ) )
		MemoryError( "tempRow", "readDataFromStream" );
	if( !( tempSmiles = calloc( maxNumberOfCharsPerRow + 2, sizeof( char ) ) ) )
		MemoryError( "tempSmiles", "readDataFromStream" );
	if( !( tempName = calloc( maxNumberOfCharsPerRow + 2, sizeof( char ) ) ) )
		MemoryError( "tempName", "readDataFromStream" );
	listPtr = List_create( uniqueData, SmilesCompound_display, SmilesCompound_destroy, \
		SmilesCompound_identical );
	sAPtr = StringArray_create( );

	/* in the following while loop each row of the file is read and stored in the strucure SmilesCompound */
	/* looping over all rows of 'inputSream' */
	while( fgets( tempRow, maxNumberOfCharsPerRow + 2, inputStream ) != NULL )
	{
		#if COUNTSMARTS_DEBUG
		fprintf( stdout, "tempRow = %s", tempRow ); fflush( NULL );
		#endif

		/* the input file contains only one column - the SMILES string */
		if( numberOfColumns == 1 )
		{
			strncpy( tempSmiles, tempRow, strlen( tempRow ) - 1 );
			tempSmiles[ strlen( tempRow ) - 1 ] = '\0';
			strncpy( tempName, tempRow, strlen( tempRow ) - 1 );
			tempName[ strlen( tempRow ) - 1 ] = '\0';
		}
		/* the input file contains two columns - the SMILES string and the name */
		else
		{
			substring = strstr( tempRow, "\t" );
			strncpy( tempSmiles, tempRow, ( substring - tempRow ) );
			tempSmiles[ substring - tempRow ] = '\0';
			strncpy( tempName, substring + 1, ( tempRow + strlen( tempRow ) - 1 ) - ( substring + 1 ) );
			tempName[ ( tempRow + strlen( tempRow ) - 1 ) - ( substring + 1 ) ] = '\0';
		}
		#if COUNTSMARTS_DEBUG
		fprintf( stdout, "Smiles: %s\tName: %s\n", tempSmiles, tempName ); fflush( NULL );
		#endif
		/* a SmilesCompound is created and inserted at the tail of the list
		if the insertion fails due to the presence of an identical SmilesCompound it is destroyed */
		scPtr = SmilesCompound_create( tempName, tempSmiles, daylight_type, errorLogFile );
		/* checking if the creation of the SmilesCompound was successful */
		if( scPtr != BOOLEAN_FALSE )
		{
			if( List_insertTail( listPtr, scPtr ) == BOOLEAN_FALSE )
				SmilesCompound_destroy( scPtr );
		}
		else
		{
			if( !( errorMessage = calloc( 300, sizeof( char ) ) ) )
				MemoryError( "errorMessage", "readDataFromStream" );
			sprintf( errorMessage, "Error reading compound %s\n", tempName );
			StringArray_addElement( sAPtr, errorMessage );
		}
	}
	/* if there are any error messages they are printed to stderr */
	if( StringArray_getNumberOfElements( sAPtr ) != 0 )
		StringArray_display( sAPtr, stderr );
	
	/* cleaning up */
	free( tempRow );
	free( tempSmiles );
	free( tempName );
	StringArray_destroy( sAPtr );
	
	return listPtr;
}

/** Examines the properties of a file stream.
* 
* The number of columns, characters per column and the number of rows are counted and the result
* of this examination are stored in a structure @ref DoubleArray. This @ref DoubleArray contains
* three elements regarded to the properties of dataFile:
*
* @li index 0 - number of columns
* @li index 1 - maximum number of characters per column
* @li index 2 - number of rows
*
* The file is also checked for integrity, i.e. it is checked, if it has the same number of columns in
* all rows. If this is not the case, the define @ref FileIntegrityError is called and the program
* aborts.
*
* @param inputStream file pointer of the stream to examine
* @param nameOfStream string indicating the name of the stream
* @retval DoubleArrayPtr contains the properties of @c inputStream
* @author Uli Fechner
* @version 28/11/2003 - Uli Fechner - initial release
*/
DoubleArrayPtr getFileProperties( FILE* inputStream, const char* const nameOfStream )
{
	int numberOfColumns = 1; /* number of columns in dataFile */
	int maxCharsPerRow = 0; /* maximum number of characters per row in dataFile */
	int numberOfRows = 0; /* number of rows in dataFile */
	char currentCharacter; /* temporary storage of the character read by fgetc */
	int position; /* counter for the position in tempRow */
	char* tempRow; /* temporary string containing one row for analysis */
	char* substring; /* pointer needed for the 'number of occurences' algorithm */
	int counter = 1; /* counter needed by the 'number of occurences' algorithm */
	/* dAPtr points on a DoubleArray returning the results.
	The DoubleArray contains three values regarding to the properties of dataFile:
	index 0 - number of columns
	index 1 - maximum number of characters per column
	index 2 - number of rows */
	DoubleArrayPtr dAPtr; /* the file properties are stored in a DoubleArray structure */
	
	if( !( tempRow = calloc( 10000, sizeof( char ) ) ) )
		MemoryError( "tempRow", "getFileProperties" );	

	dAPtr = DoubleArray_create( 3 ); /* create a new DoubleArray with 3 elements */
	
	/* looping over all rows of the file */
	while( ( currentCharacter = fgetc( inputStream ) ) != EOF )
	{
		/* putting the character back in the stream */
		ungetc( currentCharacter, inputStream );
		
		/* resetting the position counter and the tab counter */
		position = 0;
		counter = 1;
		
		/* getting a line of inputStream */
		while( ( currentCharacter = fgetc( inputStream ) ) != '\n' )
		{
			tempRow[ position ] = currentCharacter;
			position++;
		}
		tempRow[ position ] = '\0';
		numberOfRows++;
		
		/* counting the columns by counting the number of occurences of '\t' */
		substring = strstr( tempRow, "\t" );
		while( substring != NULL)
		{
			counter++;			
			substring = strstr( substring + 1, "\t" );
		}
		
		/* if the number of columns of the first row are counted the variable 'numberOfColumns' is set */
		if( numberOfRows == 1 )
			numberOfColumns = counter;
		
		/* if the number of columns in one row is different from that in the first row the program aborts */
		if( counter != numberOfColumns )			
			FileIntegrityError( nameOfStream, numberOfRows, counter, numberOfColumns );
			
		/* keeping the maximum number of chars in one row up to date */
		if( strlen( tempRow ) > maxCharsPerRow )
			maxCharsPerRow = position;
	}

	/* checking if there are exactly 2 columns; if not the input file is invalid and the program aborts */
	if( numberOfColumns > 2 )
	{
		fprintf( stderr, "\nERROR: The number of columns in %s is neither one nor two!\n", nameOfStream );
		fprintf( stderr, "Only input files with the name in the first column and the SMILES string\n" );
		fprintf( stderr, "in the second column or the SMILES string as the only column are valid!\n" );
		AbortProgram;
	}
	
	/* the values regarding to the file property are copied to the struct DoubleArray */
	DoubleArray_setValue( dAPtr, 0, numberOfColumns );
	DoubleArray_setValue( dAPtr, 1, maxCharsPerRow );
	DoubleArray_setValue( dAPtr, 2, numberOfRows );
	
	free( tempRow );
	
	return dAPtr;
}

/** Displays the help text on standard error.
*
* @author UF
* @version 22/04/04 - UF - initial release
*/
void displayHelpText( )
{
	fprintf( stderr, "\nNAME:\n" );
 	fprintf( stderr, "  countSmarts\n" );
 	fprintf( stderr, "\nFUNCTION:\n" );
 	fprintf( stderr, "  Counts the occurrences of SMARTS in a SMILES.\n" );
	fprintf( stderr, "\nUSAGE:\n" );
	fprintf( stderr, "  countSmarts [Options] <INFILE\n" );
	fprintf( stderr, "     INFILE has to be a file with one SMILES per row or a tab separated file\n" );
	fprintf( stderr, "     with SMILES in the first and labels in the second column.\n" );
	fprintf( stderr, "\nREMARK:\n" );
	fprintf( stderr, "  An error log file is automatically created. Its name is the suffix '.log'\n" );
	fprintf( stderr, "  appended to the name of the output file.\n" );
	fprintf( stderr, "\nOPTIONS:\n" );
	fprintf( stderr, "  -h\n" );
	fprintf( stderr, "     Display this help text.\n" );
	fprintf( stderr, "  -v\n" );
	fprintf( stderr, "     Display detailed version information and exit.\n" );
	fprintf( stderr, "  -o FILENAME @ MANDATORY\n" );
	fprintf( stderr, "     Set FILENAME as the name of the output file.\n" );
	fprintf( stderr, "  -s FILENAME @ MANDATORY\n" );
	fprintf( stderr, "     Set FILENAME as the name of the SMARTS file (same format as the INFILE).\n" );
	fprintf( stderr, "  -m INTEGER @ [1,3] @ DEFAULT = 2\n" );
	fprintf( stderr, "     Specifies the SMARTS match type:\n" );
	fprintf( stderr, "     1 = exhaustive search [dt_match,0]\n" );
	fprintf( stderr, "     2 = unique set-of-atoms in results [dt_umatch,0])\n" );
	fprintf( stderr, "     3 = each atom appears in exactly one result [dt_xmatch,0]\n" );
	fprintf( stderr, "  -u\n" );
	fprintf( stderr, "     Only unique SMILES are written to the output file.\n" );
	fprintf( stderr, "\nAUTHOR:\n" );
	fprintf( stderr, "  Uli Fechner\n" );
	fprintf( stderr, "\nVERSION & RELEASE DATE:\n" );
 	fprintf( stderr, "  %s ( %s )\n", COUNTSMARTS_VERSION, COUNTSMARTS_DATE );
 	fprintf( stderr, "\nBUGS:\n" );
 	fprintf( stderr, "  Please report bugs to u.fechner@chemie.uni-frankfurt.de\n\n" );
	exit( EXIT_SUCCESS );
}

/** Displays the version information text on standard error.
*
* @author Uli Fechner
* @version 28/11/2003 - Uli Fechner - initial release
*/
void displayVersionInformation( void )
{
	fprintf( stderr, "\nVERSION & RELEASE DATE:\n\n");
	fprintf( stderr, "  includes.h: %s [%s]\n", INCLUDES_VERSION, INCLUDES_DATE );
	fprintf( stderr, "  generalDefines.h: %s [%s]\n", GENERALDEFINES_VERSION, GENERALDEFINES_DATE );
	fprintf( stderr, "  givenClp.c: %s [%s]\n", GIVENCLP_VERSION, GIVENCLP_DATE );
	fprintf( stderr, "  doubleArray.c: %s [%s]\n", DOUBLEARRAY_VERSION, DOUBLEARRAY_DATE );
	fprintf( stderr, "  clp.c: %s [%s]\n", CLP_VERSION, CLP_DATE );
	fprintf( stderr, "  smilesCompound.c: %s [%s]\n", SMILESCOMPOUND_VERSION, SMILESCOMPOUND_DATE );
	fprintf( stderr, "  doubleLinkedList.c: %s [%s]\n", DOUBLELINKEDLIST_VERSION, DOUBLELINKEDLIST_DATE );
	fprintf( stderr, "  countSmarts core: %s [%s]\n\n", COUNTSMARTS_VERSION, COUNTSMARTS_DATE );
	exit( EXIT_SUCCESS );
}
