/* just some information about this file */
#define CLP_VERSION	"1.0.1"
#define CLP_DATE "09-JUN-2004"

/* structures and typedefs */

/** @file
* @anchor CommandLineParameters
* Fault-tolerant structure to store the values of all command line arguments.
*
* This structure provides a fault-tolerant way to store the values of all command line arguments. The
* values are set to their default values during the creation of the structure via @ref CLP_create.
* The memory allocation of the members of the structure is managed automatically.
*
* The structure is created with @ref CLP_create, destroyed with @ref CLP_destroy and displayed with
* @ref CLP_display. With CLP_set<i>nameOfVariable</i> and CLP_get<i>nameOfVariable</i> the values of
* the individual members of the structure @ref CommandLineParameters could be set or gotten.
*
* Whenever a string is set the provided string is @b copied via @c strncpy. Hence, the string provided
* as an argument has to be freed by the caller of the function.
* 
* <B>Description of the members:</B><BR><BR>
* @c outputFile - string containing the name of the output file.<BR>
* @c reactionFile - string storing the name of the reaction file.<BR>
* @c errorLogFile - string storing the name of the SMILES error log file.<BR>
* <P>
* @author Uli Fechner
* @version 05/13/2003 - Uli Fechner - v1.0.0 - initial release
* @version 09/06/2003 - Uli Fechner - v1.0.1 - modified according to the needs of countSmarts
* @@code
*/
typedef struct
{
	char* outputFile;
	char* smartsFile;
	char* errorLogFile;
	int matchingType;
	int uniqueSmiles;
} CommandLineParameters;
/** @endcode */

/** A pointer to structure @ref CommandLineParameters is assigned the name @c CLP_Ptr. */
typedef CommandLineParameters* CLP_Ptr;

/* function prototypes */

CLP_Ptr CLP_create( void );

void CLP_destroy( CLP_Ptr clpPtr );

void CLP_display( const CLP_Ptr clpPtr, FILE* outputStream );

void CLP_setOutputFile( const CLP_Ptr clpPtr, const char* const name );

char* CLP_getOutputFile( const CLP_Ptr clpPtr );

void CLP_setSmartsFile( const CLP_Ptr clpPtr, const char* const name );

char* CLP_getSmartsFile( const CLP_Ptr clpPtr );

void CLP_setErrorLogFile( const CLP_Ptr clpPtr, const char* const name );

char* CLP_getErrorLogFile( const CLP_Ptr clpPtr );

void CLP_setMatchingType( const CLP_Ptr clpPtr, const int matchingType );

int CLP_getMatchingType( const CLP_Ptr clpPtr );

void CLP_setUniqueSmiles( const CLP_Ptr clpPtr, const int boolean );

int CLP_getUniqueSmiles( const CLP_Ptr clpPtr );

/* functions */

/** Creates a @ref CommandLineParameters structure.
*
* The structure @ref CommandLineParameters is created. The memory of the structure itself and the members
* of the structure is allocated automatically. The values of the individual command line parameters are
* set to default values as written in the help text.
* @retval CLP_Ptr pointer on the newly created structure @ref CommandLineParameters
* @author Uli Fechner
* @version 13/05/2003 - Uli Fechner - initial release
* @version 09/06/2004 - Uli Fechner - adjusted to the needs of countSmarts
*/
CLP_Ptr CLP_create( void )
{
	CLP_Ptr clpPtr;
	if( !( clpPtr = calloc( 1, sizeof( CommandLineParameters ) ) ) )
		MemoryError( "CLP_create", "clpPtr" );

	CLP_setOutputFile( clpPtr, NULL );
	CLP_setSmartsFile( clpPtr, NULL );
	CLP_setErrorLogFile( clpPtr, NULL );
	CLP_setMatchingType( clpPtr, 2 );
	CLP_setUniqueSmiles( clpPtr, BOOLEAN_FALSE );

	return clpPtr;
}

/** Destroys a structure @ref CommandLineParameters.
*
* The structure @ref CommandLineParameters the pointer @c clpPtr refers to is destroyed. All
* allocated memory of the structure and its members is automatically freed.
*
* @attention
* The two strings (@c outputFile and @c catsTypeExpression) are freed here, too.
*
* @param clpPtr pointer on the structure @ref CommandLineParameters that should be destroyed
* @author Uli Fechner
* @version 13/05/2003 - Uli Fechner - initial release
* @version 09/06/2004 - Uli Fechner - adjusted to the needs of countSmarts
*/
void CLP_destroy( CLP_Ptr clpPtr )
{
	if( clpPtr != NULL )
	{
		if( CLP_getOutputFile( clpPtr ) != NULL )
			free( CLP_getOutputFile( clpPtr ) );
		if( CLP_getSmartsFile( clpPtr ) != NULL )
			free( CLP_getSmartsFile( clpPtr ) );
		if( CLP_getErrorLogFile( clpPtr ) != NULL )
			free( CLP_getErrorLogFile( clpPtr ) );
		free( clpPtr );
	}
	else
	{
		fprintf( stderr, "\n\nERROR: Function 'CLP_destroy' is not able to destroy a structure\n" );
		fprintf( stderr, "'CommandLineParameters' that has not been created before!\n" );
		AbortProgram;
	}
}

/** Displays a structure @ref CommandLineParameters.
*
* The structure @ref CommandLineParameters the pointer @c clpPtr refers to is displayed on the FILE* @c
* outputStream.
*
* @param clpPtr pointer on the structure @ref CommandLineParameters that should be displayed
* @param outputStream FILE* on the stream the output should be sent to
* @author Uli Fechner
* @version 13/05/2003 - Uli Fechner - initial release
* @version 09/06/2003 - Uli Fechner - adjusted to the needs of countSmarts
*/
void CLP_display( CLP_Ptr clpPtr, FILE* outputStream )
{	
	fprintf( outputStream, "Output file: %s\n", CLP_getOutputFile( clpPtr ) );
	fprintf( outputStream, "Error log file: %s\n", CLP_getErrorLogFile( clpPtr ) );
	fprintf( outputStream, "Smarts file: %s\n", CLP_getSmartsFile( clpPtr ) );
	fprintf( outputStream, "MatchingType: " );
	if( CLP_getMatchingType( clpPtr ) == 1 )
		fprintf( outputStream, "1 - exhaustive search [dt_match,0]\n" );
	if( CLP_getMatchingType( clpPtr ) == 2 )
		fprintf( outputStream, "2 - unique set-of-atoms in results [dt_umatch,0]\n" );
	if( CLP_getMatchingType( clpPtr ) == 3 )
		fprintf( outputStream, "3 - each atom appears in exactly one result [dt_xmatch,0]\n" );
	fprintf( outputStream, "UniqueSmiles: " );
	if( CLP_getUniqueSmiles( clpPtr ) == BOOLEAN_FALSE )
		fprintf( outputStream, "Nope\n" );
	else
		fprintf( outputStream, "Yesh\n" );
	fprintf( outputStream, "\n" );
}

/** The name of the output file is set.
*
* @attention
* The provided parameter @c name is copied. @c name is NOT freed afterwards. This has to be done by
* the caller of this function.
*
* @param clpPtr pointer on the structure @ref CommandLineParameters that contains the value
* @param name name of the output file
* @author Uli Fechner
* @version 13/05/2003 - Uli Fechner - initial release
*/
void CLP_setOutputFile( const CLP_Ptr clpPtr, const char* const name )
{
	if( name != NULL )
	{
		if( !( clpPtr->outputFile = calloc( strlen( name ) + 1, sizeof( char ) ) ) )
			MemoryError( "CLP_setOutputFile", "clpPtr->outputFile" );	
		strncpy( clpPtr->outputFile, name, strlen( name ) + 1 );
	}
	else
		clpPtr->outputFile = NULL;
}

/** The name of the output file is returned.
*
* @param clpPtr pointer on the structure @ref CommandLineParameters that contains the value
* @retval char name of the output data file
* @author Uli Fechner
* @version 13/05/2003 - Uli Fechner - initial release
*/
char* CLP_getOutputFile( const CLP_Ptr clpPtr )
{
	return clpPtr->outputFile;
}

/** The name of the error log file is set.
*
* @attention
* The provided parameter @c name is copied. @c name is NOT freed afterwards. This has to be done by
* the caller of this function.
*
* @param clpPtr pointer on the structure @ref CommandLineParameters that contains the value
* @param name name of the error log file
* @author Uli Fechner
* @version 03/12/2003 - Uli Fechner - initial release
*/
void CLP_setErrorLogFile( const CLP_Ptr clpPtr, const char* const name )
{
	if( name != NULL )
	{
		if( !( clpPtr->errorLogFile = calloc( strlen( name ) + 1, sizeof( char ) ) ) )
			MemoryError( "CLP_setErrorLogFile", "clpPtr->errorLogFile" );	
		strncpy( clpPtr->errorLogFile, name, strlen( name ) + 1 );
	}
	else
		clpPtr->errorLogFile = NULL;
}

/** The name of the error log file is returned.
*
* @param clpPtr pointer on the structure @ref CommandLineParameters that contains the value
* @retval char name of the error log file
* @author Uli Fechner
* @version 03/12/2003 - Uli Fechner - initial release
*/
char* CLP_getErrorLogFile( const CLP_Ptr clpPtr )
{
	return clpPtr->errorLogFile;
}

/** The name of the Smarts file is set.
*
* @attention
* The provided parameter @c name is copied. @c name is NOT freed afterwards. This has to be done by
* the caller of this function.
*
* @param clpPtr pointer on the structure @ref CommandLineParameters that contains the value
* @param name name of the Smarts file
* @author Tina Grabowski
* @version 29/01/2004 - Tina Grabowski - initial release
*/
void CLP_setSmartsFile( const CLP_Ptr clpPtr, const char* const name )
{
	if( name != NULL )
	{
		if( !( clpPtr->smartsFile = calloc( strlen( name ) + 1, sizeof( char ) ) ) )
			MemoryError( "CLP_setSmartsFile", "clpPtr->SmartsFile" );	
		strncpy( clpPtr->smartsFile, name, strlen( name ) + 1 );
	}
	else
		clpPtr->smartsFile = NULL;
}

/** The name of the Smarts file is returned.
*
* @param clpPtr pointer on the structure @ref CommandLineParameters that contains the value
* @retval char name of the Smarts file
* @author Tina Grabowski
* @version 29/01/2004 - Tina Grabowski - initial release
*/
char* CLP_getSmartsFile( const CLP_Ptr clpPtr )
{
	return clpPtr->smartsFile;
}

/** Sets the number of the matching type.
*
* @param clpPtr pointer on the structure @ref CommandLineParameters that contains the value
* @param matchingType the number of the matching type
* @author UF
* @version 09/06/2004 - UF - initial release
*/
void CLP_setMatchingType( const CLP_Ptr clpPtr, const int matchingType )
{
	/* checking the allowed range of matchingType */
	if( matchingType > 0 && matchingType < 4 )	
		clpPtr->matchingType = matchingType;
	else
	{
		fprintf( stderr, "\nERROR: The value of option '-m' has to be in [1,3]!\n" );
		fprintf( stderr, "Type 'countSmarts -h' for a detailed help text.\n" );
		AbortProgram;
	}
}

/** The number of the matching type is returned.
*
* @param clpPtr pointer on the structure @ref CommandLineParameters that contains the value
* @retval int the number of the matching type
* @author UF
* @version 09/06/2004 - UF - initial release
*/
int CLP_getMatchingType( const CLP_Ptr clpPtr )
{
	return clpPtr->matchingType;
}

/** Sets whether or not the input file is filtered to yield unique SMILES.
*
* @param clpPtr pointer on the structure @ref CommandLineParameters that contains the value
* @param boolean boolean value (@ref BOOLEAN_FALSE or @ref BOOLEAN_TRUE)
* @author UF
* @version 09/06/2004 - UF - initial release
*/
void CLP_setUniqueSmiles( const CLP_Ptr clpPtr, const int boolean )
{
	clpPtr->uniqueSmiles = boolean;
}

/** Returns whether or not the input file is filtered to yield unique SMILES.
*
* @param clpPtr pointer on the structure @ref CommandLineParameters that contains the value
* @retval int boolean value (@ref BOOLEAN_FALSE or @ref BOOLEAN_TRUE)
* @author UF
* @version 09/06/2004 - UF - initial release
*/
int CLP_getUniqueSmiles( const CLP_Ptr clpPtr )
{
	return clpPtr->uniqueSmiles;
}
