/* just some information about this file */
#define SMILESCOMPOUND_VERSION	"1.4.0"
#define SMILESCOMPOUND_DATE	"16-Mar-2004"

/* structures and typedefs */

/** @file
* @anchor SmilesCompound
* Fault-tolerant structure to store the data of one SMILES, SMIRKS or SMARTS.
* <BR>
* This structure stores the data of one SMILES, SMIRKS or SMARTS.
* <BR>
* The structure is created with @ref SmilesCompound_create, destroyed with @ref SmilesCompound_destroy
* and displayed with @ref SmilesCompound_display. With SmilesCompound_set<i>nameOfMember</i> and
* SmilesCompound_get<i>nameOfMember</i> the values of the individual members of the structure
* @ref SmilesCompound could be set or gotten. @ref SmilesCompound_identical checks if the @c smiles
* of two SmilesCompounds are identical (returns @ref BOOLEAN_TRUE) or non-identical (returns
* @ref BOOLEAN_FALSE). Non-Identity is only true if both @c smiles are canonical.
* <BR>
* If a @ref SmilesCompound is created with a SMILES, member @c smiles stores the canonical SMILES of
* that compound to allow for a idenditcal check (@ref SmilesCompound_identical). Canonical SMIRKS are
* not supported by the Daylight Toolkit. Hence, the uniqueness check fails, if two reactions share
* the same semantics but differ lexicographically. The same holds for SMARTS.
* <BR>
* The name of a SMILES, SMIRKS or SMARTS is stored in a @ref StringArray. Thus, it is possible to store
* more than one name for a given SMILES, SMIRKS or SMARTS. Detailed information about accessibility of
* the name is provided by the documentation of the respective functions (@ref SmilesCompound_setName,
* @ref SmilesCompound_addName and @ref SmilesCompound_getStringArrayOfNames).
* <BR>
* @attention
* If the member @c smiles or the name of a SMILES, SMIRKS or SMART is set (@ref SmilesCompound_setName,
* @ref SmilesCompound_addName and @ref SmilesCompound_setSmiles) the provided string is @b copied via
* @c strncpy. Hence, the string provided as an argument has to be freed by the caller of the function.
* 
* <BR><B>Description of the members:</B><BR><BR>
*	@c smiles - string containing the SMILES, SMIRKS or SMARTS of one compound<BR>
* @c counter - counts the occurence of a specific SMILES (does <b>not</b> work for SMIRKS and SMARTS as they cannot be converted to a canonical representation)<BR>
* @c sAPtr - pointer on structure @ref StringArray that stores one or more names of the SMILES, SMIRKS or SMARTS<BR>
* @c moleculeHandle - dt_Handle on a Daylight object that correspoinds to @c smiles<BR>
* @c fingerprintHandle - dt_Handle on a Daylight fingerprint object that corresponds to @c smiles<BR>
* @author Uli Fechner
* @version 28/11/2003 - Uli Fechner - 1.0.0 - initial release
* @version 02/12/2003 - Uli Fechner - 1.0.2 - added the member @c counter and the functions @ref
* SmilesCompound_getCounter, @ref SmilesCompound_setCounter, @ref SmilesCompound_increaseCounter
* and @ref SmilesCompound_decreaseCounter; added the member @c moleculeHandle and the functions
* @ref SmilesCompound_getMoleculeHandle and @ref SmilesCompound_setMoleculeHandle
* @version 03/12/2003 - Uli Fechner - 1.0.3 - added support for write out of errors revieved from
* dt_smilinerrors (@ref SmilesCompound_create); added function @ref SmilesCompound_identical
* @ version 04/12/2003 - Uli Fechner - 1.0.4 - @ref SmilesCompound_create stores now the canonical
* SMILES to allow for identical check between two SmilesCompouns; small change in @ref
* SmilesCompound_display
* @version 08/12/2003 - Uli Fechner - 1.0.5 - changes in @ref SmilesCompound_create to allow for the
* checking of a successful creation
* @version 10/12/2003 - Uli Fechner - 1.1.0 - changed the storage of a compound name from char* to
* @ref StringArray; modified function @ref SmilesCompound_create, @ref SmilesCompound_destroy, @ref
* SmilesCompound_display, @ref SmilesCompound_setName, @ref SmilesCompound_addName and 
* SmilesCompound_getName accordingly; change in @ref SmilesCompound_identical: if two SmilesCompounds
* are identical the @ref StringArray of the source SmilesCompound is concatenated to the @ref
* StringArray of the target compound
* @version 17/12/2003 - Uli Fechner - 1.2.0 - changed the name of SmilesCompound_getName to
* @ref SmilesCompound_getStringArrayOfNames; complete re-write of @ref SmilesCompound_setSmiles
* (changed return type from void to int); added @ref SmilesCompound_copy
* @version 19/12/2003 - Uli Fechner - 1.2.1 - added function @ref SmilesCompound_removeFromArrayOfNames;
* major bugifx in @ref SmilesCompound_identical
* @version 29/01/2004 - Tina Grabowski - 1.3.0 - major change: added support for SMARTS; corresponding
* changes in @ref SmilesCompound_create and @ref SmilesCompound_setSmiles
* @version 16/03/2004 - UF - 1.4.0 - added member fingerprint and the functions 
* SmilesCompound_getFingerprint and SmilesCompound_setFingerprint; renamed member @c daylightHandle
* to @c moleculeHandle in the complete file; renamed functions SmilesCompound_get/setDaylightHandle to
* SmilesCompound_get/setMoleculeHandle accordingly
* @@code
*/
typedef	struct
{
	char* smiles;
	int counter;
	StringArray_Ptr sAPtr;
	dt_Handle moleculeHandle;
	dt_Handle fingerprintHandle;
} SmilesCompound;
/** @endcode */

/** A pointer to structure @ref SmilesCompound is assigned the name @c SmilesCompound_Ptr. */
typedef SmilesCompound* SmilesCompound_Ptr;

/* defines */

/** Boolean variables are simulated with the defines BOOLEAN_FALSE and BOOLEAN_TRUE.
*
* @author Uli Fechner
* @version 03/12/2003 - Uli Fechner - initial release
*/
#ifndef BOOLEAN_FALSE
#define BOOLEAN_FALSE 0
#endif

/** Boolean variables are simulated with the defines BOOLEAN_FALSE and BOOLEAN_TRUE.
*
* @author Uli Fechner
* @version 03/12/2003 - Uli Fechner - initial release
*/
#ifndef BOOLEAN_TRUE
#define BOOLEAN_TRUE 1
#endif

/* define 'functions' */

/** The member @c counter of structure @ref SmilesCompound is increased by one.
*
* @param scPtr pointer on the structure @ref SmilesCompound the member @c counter belongs to
* @author Uli Fechner
* @version 02/12/2003 - Uli Fechner - initial release
*/
#ifndef SmilesCompound_increaseCounter
#define SmilesCompound_increaseCounter( scPtr ) \
	( (scPtr->counter)++)
#endif

/** The member @c counter of structure @ref SmilesCompound is decreased by one.
*
* @param scPtr pointer on the structure @ref SmilesCompound the member @c counter belongs to
* @author Uli Fechner
* @version 02/12/2003 - Uli Fechner - initial release
*/
#ifndef SmilesCompound_decreaseCounter
#define SmilesCompound_decreaseCounter( scPtr ) \
	( (scPtr->counter)--)
#endif

/* function prototypes */

SmilesCompound_Ptr SmilesCompound_create( char* const name, char* const smiles, const int daylight_type, \
	FILE* errorLogFile );

void SmilesCompound_destroy( void* scPtr );

void SmilesCompound_display( void* scPtr, FILE* outputStream );

SmilesCompound_Ptr SmilesCompound_copy( const SmilesCompound_Ptr scPtr );

int SmilesCompound_identical( const void* scPtr1, const void* scPtr2 );

void SmilesCompound_setName( const SmilesCompound_Ptr scPtr, const char* const name );

void SmilesCompound_addName( const SmilesCompound_Ptr scPtr, const char* const name );

StringArray_Ptr SmilesCompound_getStringArrayOfNames( const SmilesCompound_Ptr scPtr );

int SmilesCompound_removeFromArrayOfNames( const SmilesCompound_Ptr scPtr, const char* name );

int SmilesCompound_setSmiles( const SmilesCompound_Ptr scPtr, char* smiles, const int daylight_type, \
	FILE* errorLogFile );

char* SmilesCompound_getSmiles( const SmilesCompound_Ptr scPtr );

void SmilesCompound_setCounter( const SmilesCompound_Ptr scPtr, const int counter );

int SmilesCompound_getCounter( const SmilesCompound_Ptr scPtr );

void SmilesCompound_setMoleculeHandle( const SmilesCompound_Ptr scPtr, const dt_Handle moleculeHandle );

dt_Handle SmilesCompound_getMoleculeHandle( const SmilesCompound_Ptr scPtr );

/* functions */

/** Creates a @ref SmilesCompound structure.
*
* The structure @ref SmilesCompound is created. The memory of the structure itself and of its members
* is allocated automatically. The name and the SMILES, SMIRKS or SMARTS string have to be provided
* as arguments.
* <BR>
* The function returns a SmilesCompound_Ptr on the newly created structure @ref SmilesCompound or @ref
* BOOLEAN_FALSE if the creation of the @ref SmilesCompound fails somehow. Thus, the caller of this
* function should check for the return value @ref BOOLEAN_FALSE, to guarantee the successful creation
* of the @ref SmilesCompound.
* <BR>
* Parameter @c daylight_type may have one of the following three values:
* @li @ref DAYLIGHT_SMILES to indicate that @c smiles is a SMILES
* @li @ref DAYLIGHT_SMIRKS to indicate that @c smiles is a SMIRKS
* @li @ref DAYLIGHT_SMARTS to indicate that @c smiles is a SMARTS
* <BR>
* @attention
* The provided parameters @c name and @c smiles are copied. @c name and @c smiles are <b>not</b> freed
* afterwards. This has to be done by the caller of this function. A Daylight object of the SMILES, SMIRKS
* or SMARTS that is represented by the char* @c smiles is automatically created with the function
* dt_smilin, dt_smirkin and dt_smartin, respectively.
*
* @param name the name (i.e. identifier) of the SMILES, SMIRKS or SMARTS
* @param smiles the SMILES, SMIRKS or SMARTS string of the compound
* @param daylight_type variable to indicate whether smiles contains SMILES, SMIRKS or SMARTS
* @param errorLogFile file pointer on the error log file
* @retval SmilesCompound_Ptr pointer on the newly created structure @ref SmilesCompound or BOOLEAN_FALSE if the creation fails somehow
* @author Uli Fechner
* @version 28/11/2003 - Uli Fechner - initial release
* @version 02/12/2003 - Uli Fechner - added support for member @c counter and @c moleculeHandle;
* added boolean parameter transform
* @version 03/12/2003 - Uli Fechner - added parameter @c errorLogFile to write out errors revieved
* from dt_smilinerrors
* @version 04/12/2003 - Uli Fechner - the counter is set to one during creation; the smiles is either
* the canonical SMILES generated with dt_cansmiles in case of a SMILES or the original input in case
* of a SMIRK
* @version 08/12/2003 - Uli Fechner - if dt_cansmiles or dt_smirkin is not able to return a valid
* SMILES or transform object the function returns BOOLEAN_FALSE
* @version 10/12/2003 - Uli Fechner - reflect the change of name storage (name is no longer stored in
* a char* but in a @ref StringArray)
* @version 29/01/2004 - Tina Grabowski - major change: added support for reading SMARTS
*/
SmilesCompound_Ptr SmilesCompound_create( char* const name, char* const smiles, const int daylight_type, \
	FILE* errorLogFile )
{
	SmilesCompound_Ptr scPtr; /* points on the newly created SmilesCompound */
	char* canonicalSmiles; /* stores the result of the dt_cansmiles function */
	dt_Handle sequenceErrorHandle, stringObjectHandle; /* used to process the dt_smilin errors */
	dt_Integer length; /* needed by function dt_cansmiles and dt_smilinerrors */
	char* copyOfName; /* storage for a copy of the function parameter name */

	if( !( scPtr = calloc( 1, sizeof( SmilesCompound ) ) ) )
		MemoryError( "scPtr", "SmilesCompound_create" );
		
	scPtr->sAPtr = StringArray_create( );
	if( name != NULL )
	{
		if( !( copyOfName = calloc( strlen( name ) + 1, sizeof( char ) ) ) )
			MemoryError( "copyOfName", "SmilesCompound_create" );	
		strncpy( copyOfName, name, strlen( name ) + 1 );	
		StringArray_addElement( scPtr->sAPtr, copyOfName );
	}
	scPtr->counter = 1;
	
	/* differentiating between SMILES, SMIRKS and SMARTS */
	if( daylight_type == DAYLIGHT_SMILES )
	{
		/* generating the daylight molecule object and the canonical SMILES
		the canonical SMILES is then stored in scPtr->smiles */
		scPtr->moleculeHandle = dt_smilin( strlen( smiles ), smiles );
		canonicalSmiles = dt_cansmiles( &length, scPtr->moleculeHandle, 1 );
		if( length != -1 )
		{
			//fprintf( stderr, "length = %d\n", length );
			if( !( scPtr->smiles = calloc( length + 1, sizeof( char ) ) ) )
				MemoryError( "scPtr->smiles", "SmilesCompound_create" );
			strncpy( scPtr->smiles, canonicalSmiles, length + 1 );
		}
		else
		{
			StringArray_destroy( scPtr->sAPtr );
			free( scPtr );
			return BOOLEAN_FALSE;
		}
	}
	/* we are dealing with SMIRKS */
	if( daylight_type == DAYLIGHT_SMIRKS )
	{
		scPtr->moleculeHandle = dt_smirkin( strlen( smiles ), smiles );
		if( scPtr->moleculeHandle != NULL_OB )
		{
			/* scPtr->smiles is assigned to the input SMIRKS */
			if( !( scPtr->smiles = calloc( strlen( smiles ) + 1, sizeof( char ) ) ) )
				MemoryError( "scPtr->smiles", "SmilesCompound_create" );	
			strncpy( scPtr->smiles, smiles, strlen( smiles ) + 1 );	
		}
		else
		{
			StringArray_destroy( scPtr->sAPtr );
			free( scPtr );
			return BOOLEAN_FALSE;
		}
	}
	/* we are dealing with SMARTS */
	if( daylight_type == DAYLIGHT_SMARTS )
	{
		scPtr->moleculeHandle = dt_smartin( strlen( smiles ), smiles );
		if( scPtr->moleculeHandle != NULL_OB )
		{
			/* scPtr->smiles is assigned to the input SMARTS */
			if( !( scPtr->smiles = calloc( strlen( smiles ) + 1, sizeof( char ) ) ) )
				MemoryError( "scPtr->smiles", "SmilesCompound_create" );	
			strncpy( scPtr->smiles, smiles, strlen( smiles ) + 1 );	
		}
		else
		{
			StringArray_destroy( scPtr->sAPtr );
			free( scPtr );
			return BOOLEAN_FALSE;
		}
	}
		
	/* writing the dt_smilin errors to the error log file */
	sequenceErrorHandle = dt_smilinerrors();
	while( ( stringObjectHandle = dt_next( sequenceErrorHandle ) ) != NULL_OB )
		fprintf( errorLogFile, "%s\n", dt_stringvalue( &length, stringObjectHandle ) );		
	
	return scPtr;
}


/** Destroys a @ref SmilesCompound structure.
*
* The structure @ref SmilesCompound the pointer @c scPtr refers to is destroyed. The memory of the
* structure itself and of its members is freed automatically.
*
* @param scPtr pointer on structure @ref SmilesCompound that should be destroyed
* @author Uli Fechner
* @version 28/11/2003 - Uli Fechner - initial release
* @version 02/12/2003 - Uli Fechner - added support for the member @c moleculeHandle
* @version 10/12/2003 - Uli Fechner - reflect the change of name storage (name is no longer stored in
* a char* but in a @ref StringArray)
*/
void SmilesCompound_destroy( void* scPtr )
{
	SmilesCompound_Ptr castedScPtr;
	castedScPtr = (SmilesCompound_Ptr) scPtr;
	
	if( castedScPtr != NULL )
	{
		if( castedScPtr->sAPtr != NULL )
			StringArray_destroy( castedScPtr->sAPtr );
		if( castedScPtr->smiles != NULL )
			free( castedScPtr->smiles );
		if( castedScPtr->moleculeHandle != NULL_OB )
			dt_dealloc( castedScPtr->moleculeHandle );
		free( castedScPtr );
	}
	else
	{
		fprintf( stderr, "\n\nERROR: Function 'SmilesCompound_destroy' could not destroy a structure\n" );
		fprintf( stderr, "'SmilesCompound' that has not been created before!\n" );
		AbortProgram;
	}
}

/** Displays a structure @ref SmilesCompound.
*
* The structure @ref SmilesCompound the pointer @c scPtr refers to is displayed on the FILE* @c
* outputStream.
*
* @param scPtr pointer on the structure @ref SmilesCompound that should be displayed
* @param outputStream FILE* on the stream the output should be sent to
* @author Uli Fechner
* @version 28/11/2003 - Uli Fechner - initial release
* @version 02/12/2003 - Uli Fechner - added support for member @c counter and @c moleculeHandle
* @version 04/12/2003 - Uli Fechner - member @c counter is now displayed regardless of the value;
* revoked support for @c moleculeHandle as from now on the canonical smiles is stored in @c smiles
* @version 10/12/2003 - Uli Fechner - removed output of dt_typename; changed order of output; reflect
* the change of name storage (name is no longer stored in a char* but in a @ref StringArray)
*/
void SmilesCompound_display( void* scPtr, FILE* outputStream )
{
	SmilesCompound_Ptr castedScPtr;
	castedScPtr = (SmilesCompound_Ptr) scPtr;
		
	fprintf( outputStream, "%s", SmilesCompound_getSmiles( scPtr ) );
	fprintf( outputStream, "\t%d", SmilesCompound_getCounter( scPtr ) );
	if( StringArray_getNumberOfElements( castedScPtr->sAPtr ) != 0 )
	{
		fprintf( outputStream, "\t" );
		StringArray_display( castedScPtr->sAPtr, outputStream );
	}
	fprintf( outputStream, "\n" );
}

/** Returns a deepcopy of the @ref SmilesCompound @c scPtr.
*
* @param scPtr pointer on the @ref SmilesCompound that is copied
* @retval SmilesCompound_Ptr a deepcopy of the @ref SmilesCompound @c scPtr
* @author Uli Fechner
* @version 17/12/2003 - Uli Fechner - initial release
*/
SmilesCompound_Ptr SmilesCompound_copy( const SmilesCompound_Ptr scPtr )
{
	SmilesCompound_Ptr copyOfScPtr; /* copy of the SmilesCompound scPtr */	
	char* copyOfSmilesString; /* storage for a copy of the smiles string scPtr->smiles */

	if( !( copyOfScPtr = calloc( 1, sizeof( SmilesCompound ) ) ) )
		MemoryError( "copyOfScPtr", "SmilesCompound_copy" );
		
	if( scPtr->smiles != NULL )
	{
		if( !( copyOfSmilesString = calloc( strlen( scPtr->smiles ) + 1, sizeof( char ) ) ) )
			MemoryError( "copyOfSmilesString", "SmilesCompound_copy" );
		strncpy( copyOfSmilesString, scPtr->smiles, strlen( scPtr->smiles ) + 1 );
	}
	copyOfScPtr->counter = scPtr->counter;
	copyOfScPtr->sAPtr = StringArray_copy( scPtr->sAPtr );
	copyOfScPtr->moleculeHandle = dt_copy( scPtr->moleculeHandle );
	
	return copyOfScPtr;
}

/** Checks if the two SmilesCompounds @c scPtr1 and @c scPtr2 have identical char* @c smiles.
*
* If the two SmilesCompounds @c scPtr1 and @c scPtr2 have identical char* @c smiles, the @c counter
* of @c scPtr1 is increased by one and a copy of the @ref StringArray of @c scPtr2 is concatenated
* to the @ref StringArray @c sAPtr.
*
* @param scPtr1 pointer on the first structure @ref SmilesCompound
* @param scPtr2 pointer on the second structure @ref SmilesCompound
* @retval int BOOLEAN_TRUE if the smiles of @c scPtr1 and @c scPtr2 are identical, BOOLEAN_FALSE otherwise
* @author Uli Fechner
* @version 03/12/2003 - Uli Fechner - initial release
* @version 10/12/2003 - Uli Fechner - if the @c smiles of two compounds are identical, a copy of
* StringArray of @c scPtr2 is concatenated to the StringArray of @c scPtr1
* @version 19/12/2003 - Uli Fechner - major bugifx: the counter is now updated correctly
*/
int SmilesCompound_identical( const void* scPtr1, const void* scPtr2 )
{
	if( strcmp( ( (SmilesCompound_Ptr) scPtr1)->smiles, ( (SmilesCompound_Ptr) scPtr2)->smiles ) != 0 )
		return BOOLEAN_FALSE;
	
	SmilesCompound_setCounter( ( SmilesCompound_Ptr ) scPtr1, \
		SmilesCompound_getCounter( ( SmilesCompound_Ptr ) scPtr1 ) + \
		SmilesCompound_getCounter( ( SmilesCompound_Ptr ) scPtr2 ) );
	StringArray_concatenate( ( (SmilesCompound_Ptr) scPtr1)->sAPtr, ( (SmilesCompound_Ptr) scPtr2)->sAPtr );
	return BOOLEAN_TRUE;
}

/** The name of structure @ref SmilesCompound is set to @c name as the only name.
*
* The member @c sAPtr is destroyed and newly created with @c name as the only name. Thus, all
* names that were stored previously are lost. If @c name should be added to the list of names,
* function @ref SmilesCompound_addName has to be used.
*
* @attention
* The provided parameter @c name is copied. @c name is not freed afterwards. This has to be done by
* the caller of this function. @c name is not set as the name of @c scPtr if it is NULL.
*
* @param scPtr pointer on the structure @ref SmilesCompound the member @c name is set
* @param name name of the compound
* @author Uli Fechner
* @version 28/11/2003 - Uli Fechner - initial release
* @version 10/12/2003 - Uli Fechner - complete rewrite to reflect the change of name storage (name
* is no longer stored in a char* but in a @ref StringArray)
*/
void SmilesCompound_setName( const SmilesCompound_Ptr scPtr, const char* const name )
{
	char* tempString;
	
	if( scPtr->sAPtr != NULL )
		StringArray_destroy( scPtr->sAPtr );
	
	scPtr->sAPtr = StringArray_create( );
	if( strlen( name ) != 0 )
	{
		if( !( tempString = calloc( strlen( name ) + 1, sizeof( char ) ) ) )
			MemoryError( "tempString", "SmilesCompound_setName" );	
		strncpy( tempString, name, strlen( name ) + 1 );
		StringArray_addElement( scPtr->sAPtr, tempString );
	}
}

/** @c name is added to the list of names of structure @ref SmilesCompound.
*
* If @c name should be set as the only name of @c scPtr, function @ref SmilesCompound_setName has
* to be used. If names are added with this function they are preceeded by a tab character to separate
* the individual names from each other in the output.
*
* @attention
* The provided parameter @c name is copied. @c name is not freed afterwards. This has to be done by
* the caller of this function. @c name is not added to the list of names of @c scPtr if it is NULL.
*
* @param scPtr pointer on the structure @ref SmilesCompound the member @c name is added
* @param name name of the compound
* @author Uli Fechner
* @version 10/12/2003 - Uli Fechner - initial release
*/
void SmilesCompound_addName( const SmilesCompound_Ptr scPtr, const char* const name )
{
	char* tempString;
	
	if( scPtr->sAPtr == NULL )
		scPtr->sAPtr = StringArray_create( );
	
	if( strlen( name ) != 0 )
	{
		if( !( tempString = calloc( strlen( name ) + 2, sizeof( char ) ) ) )
			MemoryError( "tempString", "SmilesCompound_addName" );
		sprintf( tempString, "\t" );
		strncat( tempString, name, strlen( name ) + 1 );
		StringArray_addElement( scPtr->sAPtr, tempString );
	}
}

/** The member @c sAPtr (contains the name(s)) of structure @ref SmilesCompound is returned.
*
* @param scPtr pointer on the structure @ref SmilesCompound the member @c name belongs to
* @retval StringArray_Ptr pointer on structure StringArray that contains the name(s) of the compound
* @author Uli Fechner
* @version 28/11/2003 - Uli Fechner - initial release
* @version 10/12/2003 - Uli Fechner - reflect the change of name storage (name is no longer stored in
* a char* but in a @ref StringArray)
* @version 17/12/2003 - Uli Fechner - changed the function name from SmilesCompound_getName to
* SmilesCompound_getStringArrayOfNames
*/
StringArray_Ptr SmilesCompound_getStringArrayOfNames( const SmilesCompound_Ptr scPtr )
{
	return scPtr->sAPtr;
}

/** All names of @c scPtr that are identical to @c name are removed from the @ref StringArray that contains the names.
*
* @attention
* The @c counter of the respective @ref SmilesCompound referred to by @c scPtr is automatically adjusted
* as follows: newValue = oldValue - occurrencesOfName.
*
* @param scPtr pointer on the structure @ref SmilesCompound that contains the array of names
* @param name the name that is looked for in the @ref StringArray of @c scPtr
* @retval int the number of occurences of @c name in @c scPtr's array of names
* @author Uli Fechner
* @version 19/12/2003 - Uli Fechner - initial release
*/
int SmilesCompound_removeFromArrayOfNames( const SmilesCompound_Ptr scPtr, const char* name )
{
	int occurrencesOfName;
	
	occurrencesOfName = StringArray_removeElement( SmilesCompound_getStringArrayOfNames( scPtr ), name );
	SmilesCompound_setCounter( scPtr, SmilesCompound_getCounter( scPtr ) - occurrencesOfName );
	
	return occurrencesOfName;
}

/** The member @c smiles of structure @ref SmilesCompound is set.
*
* If the function is called the previous @c smiles and @c moleculeHandle are destroyed. A new Daylight
* object is created with dt_smilin, dt_smirkin or dt_smartin if a SMILES, SMIRKS or SMARTS is provided,
* respectively. In case of a SMILES the given @c smiles is converted to a canonical representation with
* dt_cansmiles. If dt_smilin or dt_cansmiles fails somehow, the complete @ref SmilesCompound is destroyed
* and the function returns @ref BOOLEAN_FALSE. If everything goes smoothly @ref BOOLEAN_TRUE is returned.
*
* @attention
* The provided parameter @c smiles is copied. @c smiles is not freed afterwards. This has to be done by
* the caller of this function.<BR>
*
* @param scPtr pointer on the structure @ref SmilesCompound the member @c smiles is set
* @param smiles SMILES, SMIRKS or SMARTS string
* @param daylight_type variable to indicate whether smiles contains SMILES, SMIRKS or SMARTS
* @param errorLogFile file pointer on the error log file
* @author Uli Fechner
* @version 28/11/2003 - Uli Fechner - initial release
* @version 17/12/2003 - Uli Fechner - re-written from scratch; now, provides consistency between @c
* smiles and @c moleculeHandle
* @version 29/01/2004 - Tina Grabowski - major change: added support for SMIRKS and SMARTS as @ref
*                       SmilesCompound_create does
*/
int SmilesCompound_setSmiles( const SmilesCompound_Ptr scPtr, char* smiles, const int daylight_type, \
	FILE* errorLogFile)
{
	dt_Integer lengthCanSmiles; /* required by dt_cansmiles */
	dt_Integer lengthSmilinError; /* required by dt_stringvalue */
	char* canonicalSmiles; /* stores the canonical SMILES returned by dt_smilin */
	dt_Handle sequenceErrorHandle, stringObjectHandle; /* used to process the dt_smilin errors */
	
	if( scPtr->smiles != NULL )
		free( scPtr->smiles );	
	if( scPtr->moleculeHandle != NULL_OB )
		dt_dealloc( scPtr->moleculeHandle );
	
	/* we are dealing with a SMILES */
	if( daylight_type == DAYLIGHT_SMILES )
	{
		/* generating the daylight molecule object and the canonical SMILES
		the canonical SMILES is then stored in scPtr->smiles */
		scPtr->moleculeHandle = dt_smilin( strlen( smiles ), smiles );
		canonicalSmiles = dt_cansmiles( &lengthCanSmiles, scPtr->moleculeHandle, 1 );
		/* testing whether the call of dt_cansmiles was successful */
		if( lengthCanSmiles != -1 )
		{
			if( !( scPtr->smiles = calloc( lengthCanSmiles + 1, sizeof( char ) ) ) )
				MemoryError( "scPtr->smiles", "SmilesCompound_setSmiles" );
			strncpy( scPtr->smiles, canonicalSmiles, lengthCanSmiles + 1 );
		}
		else
		{
			SmilesCompound_destroy( scPtr );
			return BOOLEAN_FALSE; //FUNCTION EXITS HERE
		}
	}
	/* we are dealing with a SMIRKS */
	if( daylight_type == DAYLIGHT_SMIRKS )
	{
		scPtr->moleculeHandle = dt_smirkin( strlen( smiles ), smiles );
		if( scPtr->moleculeHandle != NULL_OB )
		{
			/* scPtr->smiles is assigned to the input SMIRKS */
			if( !( scPtr->smiles = calloc( strlen( smiles ) + 1, sizeof( char ) ) ) )
				MemoryError( "scPtr->smiles", "SmilesCompound_setSmiles" );	
			strncpy( scPtr->smiles, smiles, strlen( smiles ) + 1 );	
		}
		else
		{
			StringArray_destroy( scPtr->sAPtr );
			free( scPtr );
			return BOOLEAN_FALSE; //FUNCTION EXITS HERE
		}
	}
	/* we are dealing with a SMARTS */
	if( daylight_type == DAYLIGHT_SMARTS )
	{
		/* generating the daylight object; the SMARTS is then stored in scPtr->smiles */
		scPtr->moleculeHandle = dt_smartin( strlen( smiles ), smiles );
		/* testing whether the call of dt_smartin was successful */
		if( scPtr->moleculeHandle != NULL_OB )
		{
			if( !( scPtr->smiles = calloc( strlen( smiles )+ 1, sizeof( char ) ) ) )
				MemoryError( "scPtr->smiles", "SmilesCompound_setSmiles" );
			strncpy( scPtr->smiles, smiles, strlen( smiles ) + 1 );
		}
		else
		{
			SmilesCompound_destroy( scPtr );
			return BOOLEAN_FALSE; //FUNCTION EXITS HERE
		}
	}
	/* writing the dt_smilin errors to the error log file */
	sequenceErrorHandle = dt_smilinerrors();
	while( ( stringObjectHandle = dt_next( sequenceErrorHandle ) ) != NULL_OB )
		fprintf( errorLogFile, "%s\n", dt_stringvalue( &lengthSmilinError, stringObjectHandle ) );
	
	return BOOLEAN_TRUE;
}

/** The member @c smiles of structure @ref SmilesCompound is returned.
*
* @param scPtr pointer on the structure @ref SmilesCompound the member @c smiles belongs to
* @retval char* SMILES string of the compound
* @author Uli Fechner
* @version 28/11/2003 - Uli Fechner - initial release
*/
char* SmilesCompound_getSmiles( const SmilesCompound_Ptr scPtr )
{
	return scPtr->smiles;
}

/** The member @c counter of structure @ref SmilesCompound is set.
*
* @param scPtr pointer on the structure @ref SmilesCompound the member @c counter is set
* @param counter integer value that is assigned to the member @c counter
* @author Uli Fechner
* @version 02/12/2003 - Uli Fechner - initial release
*/
void SmilesCompound_setCounter( const SmilesCompound_Ptr scPtr, const int counter )
{
	scPtr->counter = counter;
}

/** The member @c counter of structure @ref SmilesCompound is returned.
*
* @param scPtr pointer on the structure @ref SmilesCompound the member @c counter belongs to
* @retval int value of the member @c counter 
* @author Uli Fechner
* @version 02/12/2003 - Uli Fechner - initial release
*/
int SmilesCompound_getCounter( const SmilesCompound_Ptr scPtr )
{
	return scPtr->counter;
}

/** The member @c moleculeHandle of structure @ref SmilesCompound is set.
*
* @param scPtr pointer on the structure @ref SmilesCompound the member @c moleculeHandle is set
* @param moleculeHandle dt_Handle that is assigned to the member @c moleculeHandle
* @author Uli Fechner
* @version 02/12/2003 - Uli Fechner - initial release
*/
void SmilesCompound_setMoleculeHandle( const SmilesCompound_Ptr scPtr, const dt_Handle moleculeHandle )
{
	scPtr->moleculeHandle = moleculeHandle;
}

/** The member @c moleculeHandle of structure @ref SmilesCompound is returned.
*
* @param scPtr pointer on the structure @ref SmilesCompound the member @c moleculeHandle belongs to
* @retval dt_Handle value of the member @c moleculeHandle
* @author Uli Fechner
* @version 02/12/2003 - Uli Fechner - initial release
*/
dt_Handle SmilesCompound_getMoleculeHandle( const SmilesCompound_Ptr scPtr )
{
	return scPtr->moleculeHandle;
}
